﻿using System;
using System.ComponentModel.Composition;
using System.ComponentModel.Composition.Hosting;
using System.Windows;
using CrossPlatformApplication;
using CrossPlatformApplication.Utils;
using CrossPlatformApplication.ViewManager;
using CrossPlatformApplication.ViewModel;
using MefContrib.Hosting.Interception;
using MefContrib.Hosting.Interception.Configuration;
using Microsoft.Mef.CommonServiceLocator;
using Microsoft.Practices.ServiceLocation;

namespace SilverlightApplication
{
    public partial class App
    {
        public App()
        {
            Startup += Application_Startup;
            Exit += Application_Exit;
            UnhandledException += Application_UnhandledException;

            InitializeComponent();
        }

        #region Injected properties

        [Import]
        public ChildViewManager ChildViewManager { private get; set; }

        #endregion

        private void Application_Startup(object sender, StartupEventArgs e)
        {
            // Create interception configuration
            var cfg = new InterceptionConfiguration()
                .AddAopInterception();

            var container = new CompositionContainer(new InterceptingCatalog(new DeploymentCatalog(), cfg));
            var locator = new MefServiceLocator(container);
            ServiceLocator.SetLocatorProvider(() => locator);
            container.ComposeExportedValue<IServiceLocator>(locator);
            container.ComposeParts(this);

            RootVisual = new MainPage { DataContext = locator.GetInstance<MainViewModel>() };
        }

        private void Application_Exit(object sender, EventArgs e)
        {

        }

        private void Application_UnhandledException(object sender, ApplicationUnhandledExceptionEventArgs e)
        {
            // If the app is running outside of the debugger then report the exception using
            // the browser's exception mechanism. On IE this will display it a yellow alert 
            // icon in the status bar and Firefox will display a script error.
            if (!System.Diagnostics.Debugger.IsAttached)
            {

                // NOTE: This will allow the application to continue running after an exception has been thrown
                // but not handled. 
                // For production applications this error handling should be replaced with something that will 
                // report the error to the website and stop the application.
                e.Handled = true;
                Deployment.Current.Dispatcher.BeginInvoke(delegate { ReportErrorToDOM(e); });
            }
        }

        private void ReportErrorToDOM(ApplicationUnhandledExceptionEventArgs e)
        {
            try
            {
                string errorMsg = e.ExceptionObject.Message + e.ExceptionObject.StackTrace;
                errorMsg = errorMsg.Replace('"', '\'').Replace("\r\n", @"\n");

                System.Windows.Browser.HtmlPage.Window.Eval("throw new Error(\"Unhandled Error in Silverlight Application " + errorMsg + "\");");
            }
            catch (Exception)
            {
            }
        }
    }
}
